@@ -20,7 +20,7 @@ from octue.configuration import load_service_and_app_configuration
 from octue.definitions import MANIFEST_FILENAME, VALUES_FILENAME
 from octue.exceptions import ServiceAlreadyExists
 from octue.log_handlers import apply_log_handler, get_remote_handler
-from octue.resources import service_backends
+from octue.resources import Manifest, service_backends
 from octue.runner import Runner
 from octue.utils.encoders import OctueJSONEncoder
 from twined import Twine
@@ -158,7 +158,7 @@ def run(service_config, input_dir, output_file, output_manifest_file, monitor_me
         handle_monitor_message=monitor_message_handler,
     )
 
-    click.echo(json.dumps(analysis.output_values))
+    click.echo(json.dumps(analysis.output_values, cls=OctueJSONEncoder))
 
     if analysis.output_values and output_file:
         if not os.path.exists(os.path.dirname(output_file)):
@@ -289,25 +289,61 @@ def start(service_config, revision_tag, timeout, no_rm):
 @click.option(
     "--local-path",
     type=click.Path(file_okay=False),
-    default=None,
+    default=".",
     help="The path to a directory to store the directory of diagnostics data in. Defaults to the current working "
     "directory.",
 )
-def get_crash_diagnostics(cloud_path, local_path):
+@click.option(
+    "--download-datasets",
+    is_flag=True,
+    help="If provided, download any datasets from the crash diagnostics and update their paths in the configuration and "
+    "input manifests to the new local paths.",
+)
+def get_crash_diagnostics(cloud_path, local_path, download_datasets):
     """Download crash diagnostics for an analysis from the given directory in Google Cloud Storage. The cloud path
     should end in the analysis ID.
 
     CLOUD_PATH: The path to the directory in Google Cloud Storage containing the diagnostics data.
     """
     analysis_id = storage.path.split(cloud_path)[-1]
-    local_path = os.path.join((local_path or "."), analysis_id)
+    local_path = os.path.join(local_path, analysis_id)
+
+    if download_datasets:
+        filter = None
+    else:
+        filter = lambda blob: any(
+            (
+                blob.name.endswith(f"configuration_{VALUES_FILENAME}"),
+                blob.name.endswith(f"configuration_{MANIFEST_FILENAME}"),
+                blob.name.endswith(f"input_{VALUES_FILENAME}"),
+                blob.name.endswith(f"input_{MANIFEST_FILENAME}"),
+                blob.name.endswith("questions.json"),
+            )
+        )
 
     GoogleCloudStorageClient().download_all_files(
         local_path=local_path,
         cloud_path=cloud_path,
+        filter=filter,
         recursive=True,
     )
 
+    # Update the manifests with the local paths of the datasets.
+    if download_datasets:
+        for manifest_type in ("configuration_manifest", "input_manifest"):
+            manifest_path = os.path.join(local_path, manifest_type + ".json")
+
+            if not os.path.exists(manifest_path):
+                continue
+
+            manifest = Manifest.from_file(manifest_path)
+
+            manifest.update_dataset_paths(
+                path_generator=lambda dataset: os.path.join(local_path, f"{manifest_type}_datasets", dataset.name)
+            )
+
+            manifest.to_file(manifest_path)
+
     logger.info("Downloaded crash diagnostics from %r to %r.", cloud_path, local_path)
 
 
@@ -327,9 +363,17 @@ def deploy():
 )
 @click.option("--no-cache", is_flag=True, help="If provided, don't use the Docker cache.")
 @click.option("--update", is_flag=True, help="If provided, allow updates to an existing service.")
-def cloud_run(service_config, update, no_cache):
+@click.option(
+    "--revision-tag",
+    type=str,
+    default=None,
+    help="A tag to use for this revision of the service (e.g. 1.3.7). This overrides the `OCTUE_SERVICE_REVISION_TAG` "
+    "environment variable if it's present. If this option isn't given and the environment variable isn't present, a "
+    "random 'cool name' tag is generated e.g 'curious-capybara'.",
+)
+def cloud_run(service_config, update, no_cache, revision_tag):
     """Deploy a python app to Google Cloud Run as an Octue service or digital twin."""
-    CloudRunDeployer(service_config).deploy(update=update, no_cache=no_cache)
+    CloudRunDeployer(service_config, revision_tag=revision_tag).deploy(update=update, no_cache=no_cache)
 
 
 @deploy.command()
@@ -349,7 +393,15 @@ def cloud_run(service_config, update, no_cache):
     help="If provided, skip creating and running the build trigger and just deploy a pre-built image to Dataflow",
 )
 @click.option("--image-uri", type=str, default=None, help="The actual image URI to use when creating the Dataflow job.")
-def dataflow(service_config, no_cache, update, dataflow_job_only, image_uri):
+@click.option(
+    "--revision-tag",
+    type=str,
+    default=None,
+    help="A tag to use for this revision of the service (e.g. 1.3.7). This overrides the `OCTUE_SERVICE_REVISION_TAG` "
+    "environment variable if it's present. If this option isn't given and the environment variable isn't present, a "
+    "random 'cool name' tag is generated e.g 'curious-capybara'.",
+)
+def dataflow(service_config, no_cache, update, dataflow_job_only, image_uri, revision_tag):
     """Deploy a python app to Google Dataflow as an Octue service or digital twin."""
     if bool(importlib.util.find_spec("apache_beam")):
         # Import the Dataflow deployer only if the `apache-beam` package is available (due to installing `octue` with
@@ -361,7 +413,7 @@ def dataflow(service_config, no_cache, update, dataflow_job_only, image_uri):
             "`pip install octue[dataflow]`"
         )
 
-    deployer = DataflowDeployer(service_config)
+    deployer = DataflowDeployer(service_config, revision_tag=revision_tag)
 
     if dataflow_job_only:
         deployer.create_streaming_dataflow_job(image_uri=image_uri, update=update)
