@@ -96,6 +96,10 @@ class MissingServiceID(OctueSDKException):
     """Raise when a specific ID for a service is expected to be provided, but is missing or None."""
 
 
+class InvalidServiceID(OctueSDKException):
+    """Raise when a service ID is invalid."""
+
+
 class CloudLocationNotSpecified(OctueSDKException):
     """Raise when attempting to interact with a cloud resource implicitly but the implicit details of its location are
     missing.
